SUBROUTINE KRAFTER(DISPL,RM,BETA,X1,X2,B1,B2,B3,B4,C6,C8,C10,EPS,RCUT,NSC,NATOM,NATOMTOT,ITTER,POT,AK,RK,AZ,FORCES)
      !This subroutine calculates the forces on all the
      ! atoms within a supercell of dimension NSC(1) X NSC(2) X NSC(3),
      ! for a Morse-spline-Van der Waals (MSV) pair potential
      ! (Physical Rewiev A, 23, 991 (1981) )
      ! the parameters are:
      ! a) Rm
      ! b) BETA
      ! c) X1,X2, b1,b2,b3,b4,c6,c8,c10
      ! d) EPS
      ! for a N-body potential (ZRP) fitted to an abinitio calculation 
      ! (PRL, VOLUME 63, NUMBER 20, 2244 (1989)  ) we have
      ! used the parameters:
      ! A <--> b1
      ! greek Z <---> b2
      ! p <---> b3
      ! q <---> b4
      ! ro <---> x1
      ! rc <---> x2
      ! For the Embedded atom potential (EAM), PRB 70, 184122 , we have
      ! the same parameter definitions as the above ZRP-potential + the
      ! additional parameter
      ! nc <---> Rm
      ! For the embedded atom potential of type (PAS), PRB 22, 12704 (1992),
      ! VOLUME 45, the EAM potential parameters are stored in the arrays:
      ! RK,AK
      ! BETA <--> BETA
      ! Lattice constant a <---> RM
      ! second derivative of embedding function F'' <---> X1
      ! The equillibrium density ( not given in the paper ) RHO_0 <--> X2
      ! For the Embedded atom potential (PIN), PRB 58, 11252 (1998) 
      ! We have for k=1..6 that the potential parameters denoted R_k and A_k
      ! In the paper PRB 58, 11252 (1998) here are the input parameters
      ! RK(i),AK(i) , i =1,...,6
      ! And for the parameters r_k and a_k in the same paper we have:
      ! a_k (k=1) <--> B1
      ! a_k ( k= 2) <--> B2
      ! r_k ( k= 1 ) <---> B3
      ! r_k ( k= 2 ) <---> B4
      ! For the Embedded atom potential (PAA), J. Nucl. Mat. 264, 198-205 (1999)
      ! ( Journal of Nuclear Materials, R.C Pasianot and A.M. Monti )
      ! The EAM parameters are stored in the arrays:
      ! RK,AK ( Pair potential part )
      ! RM (in paper ) <---> X1
      ! Rc (in paper ) <---> X2
      ! a ( lattice constant of reference structure in paper) <---> RM
      ! PHI_0 ( in paper ) <---> EPS
      ! a_1,a_2,a_3 ( in paper, embedded pot param ) <---> AZ(i), i=1,..,3
      ! The Embedding function has been parametrisized as as a 8:th order
      ! polynomial: F(RHO) = B1*RHO**8 + B2*RHO**7 + B3*RHO**6 + B4*RHO**5 +
      ! C6*RHO**4 + C8*RHO**3 + C10*RHO**2 + AZ(4)*RHO + BETA.
      !The input parameters are the following:
      !1) DISPL an 4-dim array, 1-element = the number of the atom in 
      !the supercel that is being displaced, the three following
      !elements are the displacement vector in direct coordinates.
      !2) SIGMA =  Lennard Jones potential parameter.
      !3) EPS =  Lennard Jones potential parameter.
      !4) NCUT = Number of supercells ((2*NCUT)**3) that sould be used
      ! in the calculation of the forces.
      !5) NSC = size of supercell
      !6) ITTER = if ITTER= 0 the poscar file read is POSCAR, otherwise
      !the file POSCARTEMP1 is read.
      !The out-puts are the following:
      !1) the array FORCES, containing the forces 

      IMPLICIT NONE
      CHARACTER(LEN=3), INTENT(IN) :: POT
      DOUBLE PRECISION,INTENT(IN) :: DISPL(4),RM,BETA,X1,X2
      DOUBLE PRECISION,INTENT(IN) :: B1,B2,B3,B4,C6,C8,C10
      DOUBLE PRECISION,INTENT(IN) :: RK(7),AK(7),AZ(4)
      DOUBLE PRECISION,INTENT(IN) :: EPS
      DOUBLE PRECISION,INTENT(IN) :: RCUT
      DOUBLE PRECISION,INTENT(OUT) :: FORCES(NATOMTOT,3)
      INTEGER,INTENT(IN) :: NSC(3),ITTER,NATOM,NATOMTOT
      CHARACTER(LEN=3) :: DUMMY
      DOUBLE PRECISION :: A, MATR(3,3),X,OM1,OM2,OM3,CONTROLL(3),NAMNARE1,NAMNARE2,DS,DS1,DS2
      DOUBLE PRECISION :: OMM1,OMM2,OMM3,tom1,tom2,tom3,EAM1,EAM2,EAM3,FACT,RKK(8),AKK(8)
      INTEGER,ALLOCATABLE :: SREMB(:)
      DOUBLE PRECISION, ALLOCATABLE ::RP1(:,:,:),RVP1(:,:),FCORR1(:,:,:),FCORR2(:,:,:)
      DOUBLE PRECISION, ALLOCATABLE ::RP2(:,:,:),RVP2(:,:),NAMNCORR1(:),NAMNCORR2(:)
      INTEGER, ALLOCATABLE :: INDUX(:)
      DOUBLE PRECISION, ALLOCATABLE :: F1(:,:),F2(:,:),FORCE(:,:),FREF(:,:),FF1(:,:),FF2(:,:)
      DOUBLE PRECISION, ALLOCATABLE :: FORCEBOUND(:,:),FF3(:,:),FF4(:,:)
      DOUBLE PRECISION :: R1(3),R2(3),LATT,VECT(3),SUM1(3),SUM2(3),R
      DOUBLE PRECISION ::RR1(3),SUMM1(3),SUMM2(3),rat,TESTSUM(2),LATTN,ENK1,ENK2
      DOUBLE PRECISION :: OMEG(3,3),MATR1(3,3),KRAFT(3),TES(3,3)
      DOUBLE PRECISION :: WW(3),WEIGHT,const,KRAFTI(3),MATRISI(3,3)
      DOUBLE PRECISION :: MATR2(3,3),KVEC(3),MATRIS(3,3)
      DOUBLE PRECISION :: MD(3,3),DIAG(3,3),SIGMA,d,d0,RR2(3),RCUTT,RCORN(3),jamn
      DOUBLE PRECISION :: R4(3),MATR3(3,3),R3(3),FW
      INTEGER :: NVECT,i,j,k,NUM,nn,n,NSUP,NCENTER,COUN,ll,NVECBOUND,pp
      INTEGER :: INDSAVE(8),i1,i2,i3,NCUT,DEG,DEG1,DEG2,DEG3
      INTEGER :: SK1,SK2,SK3,gg,vv,ANTAL
      DOUBLE PRECISION, PARAMETER :: pi = 3.14159265358979323846
      DOUBLE PRECISION, PARAMETER :: mep = 0.001
      DO i=1,6
        RKK(i) = RK(i)
        AKK(i) = AK(i)
      ENDDO
      AKK(7) = B1
      AKK(8) = B2
      RKK(7) = B3
      RKK(8) = B4
      SIGMA = BETA
      d = RM
      NCUT = 1
      const = (1.0/(2.0*pi))*sqrt(1.6021773/1.66054)*100 ! Trans ->THz
      NSUP =  (2*NCUT+1)**3
      NVECT = NSC(1)*NSC(2)*NSC(3)*NATOM
      ANTAL = NVECT*11**3 !NVECT*11**3
      
      ALLOCATE(INDUX(ANTAL))
      ALLOCATE(FCORR1(ANTAL,NVECT,3),FCORR2(ANTAL,NVECT,3))
      ALLOCATE(NAMNCORR1(NVECT),NAMNCORR2(NVECT))
      ALLOCATE(RP1(NSUP,NVECT,3))
      ALLOCATE(SREMB(NVECT))
      ALLOCATE(RP2(NSUP,NVECT,3))
      ALLOCATE(RVP1(NVECT,3))
      ALLOCATE(RVP2(NVECT,3))
      ALLOCATE(F1(NVECT,3))
      ALLOCATE(F2(NVECT,3))
      ALLOCATE(FF1(NVECT,3))
      ALLOCATE(FF2(NVECT,3))
      ALLOCATE(FF3(NVECT,3))
      ALLOCATE(FF4(NVECT,3))
      ALLOCATE(FORCE(NVECT,3))
      ALLOCATE(FREF(NVECT,3))
      !Here the lattice vectors (MATR), the atomic positions RP 
      ! are read from the POSCAR or POSCARTEMP1 file

      IF ( ITTER .EQ. 0  ) THEN
              OPEN(10,FILE='POSCAR',STATUS='OLD',ACTION='READ')
      ELSE
              OPEN(10,FILE='POSCARTEMP1',STATUS='OLD',ACTION='READ')
      ENDIF
      READ(10,*) DUMMY
      READ(10,*) LATT
      READ(10,*) MATR(1,1),MATR(1,2),MATR(1,3)
      READ(10,*) MATR(2,1),MATR(2,2),MATR(2,3)
      READ(10,*) MATR(3,1),MATR(3,2),MATR(3,3)
      READ(10,*) DUMMY
      READ(10,*) DUMMY
      NUM = NSC(1)*NSC(2)*NSC(3)*NATOM
      nn = 0
      DO j=1,NUM
            nn = nn +1
            READ(10,*) RVP1(nn,1),RVP1(nn,2),RVP1(nn,3)
      ENDDO
      close(10)
      d0 = d/LATT
      MATR = TRANSPOSE(MATR)
      !Here the the displacement is added to the original
      !atomic positions

      
      IF ( ITTER .EQ. 0  ) THEN
        RVP2 = RVP1
        RVP2(INT(DISPL(1)),1)= RVP1(INT(DISPL(1)),1) + DISPL(2)
        RVP2(INT(DISPL(1)),2)= RVP1(INT(DISPL(1)),2) + DISPL(3)
        RVP2(INT(DISPL(1)),3)= RVP1(INT(DISPL(1)),3) + DISPL(4)
      ELSE 
              OPEN(88,FILE='POSCAR',STATUS='OLD',ACTION='READ')
              READ(88,*) DUMMY
              READ(88,*) DUMMY
              READ(88,*) DUMMY
              READ(88,*) DUMMY
              READ(88,*) DUMMY
              READ(88,*) DUMMY
              READ(88,*) DUMMY
              nn = 0
              DO j=1,NUM
                     nn = nn +1
                     READ(88,*) RVP2(nn,1),RVP2(nn,2),RVP2(nn,3)
              ENDDO
              close(88)
      ENDIF
      !Here the atomic positions of the supercells sourounding
      !the original supercell of the POSCAR file are calculated
      COUN = 0
      DO i= -NCUT,NCUT
         DO j= -NCUT,NCUT
            DO k= -NCUT,NCUT
               COUN = COUN +1
               VECT(1) = i*1.0
               VECT(2) = j*1.0
               VECT(3) = k*1.0
               DO n=1,NVECT
                    RP1(COUN,n,1) = RVP1(n,1) + VECT(1)
                    RP1(COUN,n,2) = RVP1(n,2) + VECT(2)
                    RP1(COUN,n,3) = RVP1(n,3) + VECT(3)
                    RP2(COUN,n,1) = RVP2(n,1) + VECT(1)
                    RP2(COUN,n,2) = RVP2(n,2) + VECT(2)
                    RP2(COUN,n,3) = RVP2(n,3) + VECT(3)
               ENDDO
               IF ( i .EQ. 0 .AND. j .EQ. 0 .AND. k .EQ. 0 ) THEN
                         NCENTER = COUN
               ENDIF
         ENDDO
        ENDDO
      ENDDO


    
      !Here the forces on the centered supercell are calculated
        NAMNARE1 = 0.0d0
        NAMNARE2 = 0.0d0
        
      DO i=1,NVECT
          NAMNARE1 = 0.0d0
          NAMNARE2 = 0.0d0
          NAMNCORR1(i) = 0.0d0
          NAMNCORR2(i) = 0.0d0
         R1 = RP1(NCENTER,i,:)
         RR1 = RP2(NCENTER,i,:)
         SUM1(:) = 0.0d0
         SUM2(:) = 0.0d0
         SUMM1(:) = 0.0d0
         SUMM2(:) = 0.0d0
         IF ( i .EQ. 1 ) TESTSUM = 0
         ANTAL = 0
         DO i1=-5,5
         DO i2=-5,5
         DO i3=-5,5
            DO k=1,NVECT
               IF ( .not. (k .EQ. i .AND. i1 .EQ. 0 .AND. i2 .EQ. 0 .AND. i3 &
               & .EQ. 0 ) ) THEN
                  ANTAL = ANTAL+1
                  INDUX(ANTAL) = k
                  !Here the cell without displacement:
                  R2 = RP2(NCENTER,k,:)
                  R2(1) = R2(1)+i1*1.0d0
                  R2(2) = R2(2)+i2*1.0d0
                  R2(3) = R2(3)+i3*1.0d0
                  VECT = RR1 - R2
                  !Here we make sure that all the forces F(R) are calculted
                  !with the same number of force contributions
                  IF ( sqrt(DOT_PRODUCT(VECT,VECT)) .LE. RCUT ) THEN
                  R2 = RP1(NCENTER,k,:)
                  R2(1) = R2(1)+i1*1.0d0
                  R2(2) = R2(2)+i2*1.0d0
                  R2(3) = R2(3)+i3*1.0d0
                  VECT = R1 - R2
                  VECT = MATMUL(MATR,VECT)
                  R = sqrt(DOT_PRODUCT(VECT,VECT))
                  X = R*LATT/RM
                  IF ( POT .EQ. 'MSV' ) THEN
                    IF ( X .LE. X1 ) THEN
                        SUM1 = SUM1 + &
                        & 2*BETA*(VECT/R)*(exp(-2*BETA*(X-1))-exp(-BETA*(X-1)))
                    ENDIF
                    IF ( X .LT. X2 .AND. X .GT. X1 ) THEN
                          SUM1 = SUM1  &
                          & -(VECT/R)*(b2 + b3*( (X-X2)+(X-X1) ) + &
                          & b4*(X-X1)*(2*(X-X2)+(X-X1)))
                    ENDIF
                    IF ( X .GE. X2 ) THEN
                          SUM1 = SUM1 - (VECT/R)*( &
                          & 6*C6/(X**7) + 8*C8/(X**9)+10*C10/(X*11))
                    ENDIF
                    !Here the cell with displacement:
                    R2 = RP2(NCENTER,k,:)
                    R2(1) = R2(1)+i1*1.0d0
                    R2(2) = R2(2)+i2*1.0d0
                    R2(3) = R2(3)+i3*1.0d0
                    VECT = RR1 - R2
                    VECT = MATMUL(MATR,VECT)
                    R = sqrt(DOT_PRODUCT(VECT,VECT))
                    X = R*LATT/RM
                    IF ( X .LE. X1 ) THEN
                          SUMM1 = SUMM1 + &
                          & 2*BETA*(VECT/R)*(exp(-2*BETA*(X-1))-exp(-BETA*(X-1)))
                    ENDIF 
                    IF ( X .LT. X2 .AND. X .GT. X1 ) THEN
                          SUMM1 = SUMM1  &
                          & -(VECT/R)*(b2 + b3*( (X-X2)+(X-X1) ) + &
                          & b4*(X-X1)*(2*(X-X2)+(X-X1)))
                    ENDIF
                    IF ( X .GE. X2 ) THEN
                          SUMM1 = SUMM1 - (VECT/R)*( &
                          & 6*C6/(X**7) + 8*C8/(X**9)+10*C10/(X*11))
                    ENDIF
                  ENDIF
                  IF ( POT .EQ. 'BUC' ) THEN
                    SUM1(1) = SUM1(1) + (VECT(1)/R)*exp(SIGMA*(1.0-R/d0))
                    SUM1(2) = SUM1(2) + (VECT(2)/R)*exp(SIGMA*(1.0-R/d0))
                    SUM1(3) = SUM1(3) + (VECT(3)/R)*exp(SIGMA*(1.0-R/d0))
                    SUM2(1) = SUM2(1) + (VECT(1)/R)*(d0/R)**7
                    SUM2(2) = SUM2(2) + (VECT(2)/R)*(d0/R)**7
                    SUM2(3) = SUM2(3) + (VECT(3)/R)*(d0/R)**7

                    !Here the cell with displacement:
                    R2 = RP2(NCENTER,k,:)
                    R2(1) = R2(1)+i1*1.0d0
                    R2(2) = R2(2)+i2*1.0d0
                    R2(3) = R2(3)+i3*1.0d0
                    VECT = RR1 - R2
                    VECT = MATMUL(MATR,VECT)
                    R = sqrt(DOT_PRODUCT(VECT,VECT))
                    SUMM1(1) = SUMM1(1) + (VECT(1)/R)*exp(SIGMA*(1.0-R/d0))
                    SUMM1(2) = SUMM1(2) + (VECT(2)/R)*exp(SIGMA*(1.0-R/d0))
                    SUMM1(3) = SUMM1(3) + (VECT(3)/R)*exp(SIGMA*(1.0-R/d0))
                    SUMM2(1) = SUMM2(1) + (VECT(1)/R)*(d0/R)**7
                    SUMM2(2) = SUMM2(2) + (VECT(2)/R)*(d0/R)**7
                    SUMM2(3) = SUMM2(3) + (VECT(3)/R)*(d0/R)**7
                  ENDIF
                  IF ( POT .EQ. 'PAS' .OR. POT .EQ. 'PIN' .OR. POT .EQ. 'PAA' ) THEN
                          !Here the cell without displacement:
                          VECT = VECT*LATT
                          R = R*LATT
                          !Pair potential part.
                          IF ( POT .EQ. 'PAS' .OR. POT .EQ. 'PAA' ) THEN
                           DO vv=1,7
                                IF ( RK(vv)*RM-R .GE. 0.0d0 ) THEN
                                        SUM1(1) = SUM1(1)-3.0d0*AK(vv)*(VECT(1)/(R*LATT))*(RK(vv)*(RM/LATT)-R/LATT)**2
                                        SUM1(2) = SUM1(2)-3.0d0*AK(vv)*(VECT(2)/(R*LATT))*(RK(vv)*(RM/LATT)-R/LATT)**2
                                        SUM1(3) = SUM1(3)-3.0d0*AK(vv)*(VECT(3)/(R*LATT))*(RK(vv)*(RM/LATT)-R/LATT)**2
                                ENDIF
                           ENDDO
                          ELSE
                            DO vv=1,6
                                IF ( RKK(vv)-R .GE. 0.0d0 ) THEN
                                        SUM1(1) =SUM1(1)-3.0d0*AKK(vv)*(VECT(1)/(R))*(RKK(vv)-R)**2
                                        SUM1(2) =SUM1(2)-3.0d0*AKK(vv)*(VECT(2)/(R))*(RKK(vv)-R)**2
                                        SUM1(3) =SUM1(3)-3.0d0*AKK(vv)*(VECT(3)/(R))*(RKK(vv)-R)**2
                                ENDIF
                             ENDDO
                          ENDIF
                          !Embedding function part
                          IF ( POT .EQ. 'PAS' .OR. POT .EQ. 'PAA' ) THEN
                                  IF ( POT .EQ. 'PAS' ) THEN
                                        NAMNARE1 = NAMNARE1 +X1*EXP(-BETA*R/RM)*(RM/R)
                                        SUM2(1) =  SUM2(1)+EXP(-BETA*R/RM)*(BETA/R+RM/(R**2))*VECT(1)/R
                                        SUM2(2) =  SUM2(2)+EXP(-BETA*R/RM)*(BETA/R+RM/(R**2))*VECT(2)/R
                                        SUM2(3) =  SUM2(3)+EXP(-BETA*R/RM)*(BETA/R+RM/(R**2))*VECT(3)/R
                                  ELSE
                                          IF ( R .LE. RM*X1 ) THEN
                                                  NAMNARE1 = NAMNARE1 +EPS*EXP(-5.0d0*R/RM)*(RM/R)
                                                  SUM2(1) =SUM2(1)-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(1)/R
                                                  SUM2(2) =SUM2(2)-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(2)/R
                                                  SUM2(3) =SUM2(3)-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(3)/R
                                                  FCORR1(ANTAL,i,1)=-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(1)/R
                                                  FCORR1(ANTAL,i,2)=-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(2)/R
                                                  FCORR1(ANTAL,i,3)=-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(3)/R
                                                  NAMNCORR1(i) =NAMNCORR1(i)+EPS*EXP(-5.0d0*R/RM)*(RM/R)
                                          ENDIF
                                          IF ( R .GT. RM*X1 .AND. R .LE. RM*X2 ) THEN
                                                  NAMNARE1 = NAMNARE1+EPS*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3) )*(R/RM-X2)**3
                                                  SUM2(1) =SUM2(1)+(EPS/RM)*( (2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(1)/R
                                                  SUM2(2) =SUM2(2)+(EPS/RM)*( (2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(2)/R
                                                  SUM2(3) =SUM2(3)+(EPS/RM)*( (2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(3)/R
                                                  
                                                  FCORR1(ANTAL,i,1)=(EPS/RM)*((2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(1)/R
                                                  
                                                  FCORR1(ANTAL,i,2)=(EPS/RM)*((2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(2)/R
                                                  
                                                  FCORR1(ANTAL,i,3)=(EPS/RM)*((2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(3)/R

                                                  NAMNCORR1(i) =NAMNCORR1(i)+EPS*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**3
                                          ENDIF
                                  ENDIF
                           ELSE
                                   DO vv=7,8
                                        IF ( RKK(vv)-R .GE. 0.0d0 ) THEN
                                                NAMNARE1 = NAMNARE1+AKK(vv)*(RKK(vv)-R)**3
                                                SUM2(1) = SUM2(1)-3.0d0*AKK(vv)*(VECT(1)/(R))*(RKK(vv)-R)**2
                                                SUM2(2) = SUM2(2)-3.0d0*AKK(vv)*(VECT(2)/(R))*(RKK(vv)-R)**2
                                                SUM2(3) = SUM2(3)-3.0d0*AKK(vv)*(VECT(3)/(R))*(RKK(vv)-R)**2
                                                FCORR1(ANTAL,i,1)=FCORR1(ANTAL,i,1)-3.0d0*AKK(vv)*(VECT(1)/(R))*(RKK(vv)-R)**2
                                                FCORR1(ANTAL,i,2)=FCORR1(ANTAL,i,2)-3.0d0*AKK(vv)*(VECT(2)/(R))*(RKK(vv)-R)**2
                                                FCORR1(ANTAL,i,3)=FCORR1(ANTAL,i,3)-3.0d0*AKK(vv)*(VECT(3)/(R))*(RKK(vv)-R)**2
                                                NAMNCORR1(i) = NAMNCORR1(i)+AKK(vv)*(RKK(vv)-R)**3
                                         ENDIF
                                   ENDDO
                           ENDIF
                                                
                          
                          !Here the cell with displacement:
                          R2 = RP2(NCENTER,k,:)
                          R2(1) = R2(1)+i1*1.0d0
                          R2(2) = R2(2)+i2*1.0d0
                          R2(3) = R2(3)+i3*1.0d0
                          VECT = RR1 - R2
                          VECT = MATMUL(MATR,VECT)*LATT
                          R = sqrt(DOT_PRODUCT(VECT,VECT))
                          ! Pair potential part
                          IF ( POT .EQ. 'PAS' .OR. POT .EQ. 'PAA' ) THEN
                           DO vv=1,7
                                IF ( RK(vv)*RM-R .GE. 0.0d0 ) THEN
                                        SUMM1(1) = SUMM1(1)-3.0d0 * AK(vv)/RM**3 * (VECT(1)/R) * (RK(vv)*RM-R)**2
                                        SUMM1(2) = SUMM1(2)-3.0d0 * AK(vv)/RM**3 * (VECT(2)/R) * (RK(vv)*RM-R)**2
                                        SUMM1(3) = SUMM1(3)-3.0d0 * AK(vv)/RM**3 * (VECT(3)/R) * (RK(vv)*RM-R)**2
                                ENDIF
                           ENDDO
                          ELSE
                            DO vv=1,6
                              IF ( RKK(vv)-R .GE. 0.0d0 ) THEN
                                      SUMM1(1) = SUMM1(1)-3.0d0*AKK(vv)*(VECT(1)/(R))*(RKK(vv)-R)**2
                                      SUMM1(2) = SUMM1(2)-3.0d0*AKK(vv)*(VECT(2)/(R))*(RKK(vv)-R)**2
                                      SUMM1(3) = SUMM1(3)-3.0d0*AKK(vv)*(VECT(3)/(R))*(RKK(vv)-R)**2
                              ENDIF
                            ENDDO
                          ENDIF
                          !Embedding function part
                          IF ( POT .EQ. 'PAS' .OR. POT .EQ. 'PAA' ) THEN
				IF ( POT .EQ. 'PAS' ) THEN
                                	NAMNARE2 =  NAMNARE2 +X1*EXP(-BETA*R/RM)*(RM/R)
                                	SUMM2(1) =  SUMM2(1)+EXP(-BETA*R/RM)*(BETA/R+RM/(R**2))*VECT(1)/R
                                	SUMM2(2) =  SUMM2(2)+EXP(-BETA*R/RM)*(BETA/R+RM/(R**2))*VECT(2)/R
                                	SUMM2(3) =  SUMM2(3)+EXP(-BETA*R/RM)*(BETA/R+RM/(R**2))*VECT(3)/R
                                ELSE
                                          IF ( R .LE. RM*X1 ) THEN
                                                  NAMNARE2 = NAMNARE2 +EPS*EXP(-5.0d0*R/RM)*(RM/R)
                                                  SUMM2(1) =SUMM2(1)-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(1)/R
                                                  SUMM2(2) =SUMM2(2)-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(2)/R
                                                  SUMM2(3) =SUMM2(3)-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(3)/R
                                                  FCORR2(ANTAL,i,1)=-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(1)/R
                                                  FCORR2(ANTAL,i,2)=-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(2)/R
                                                  FCORR2(ANTAL,i,3)=-EPS*EXP(-5.0d0*R/RM)*(5.0d0/R+RM/(R**2))*VECT(3)/R
                                                  NAMNCORR2(i) =NAMNCORR2(i)+EPS*EXP(-5.0d0*R/RM)*(RM/R)
                                          ENDIF
                                          IF ( R .GT. RM*X1 .AND. R .LE. RM*X2 ) THEN
                                                  NAMNARE2 = NAMNARE2+EPS*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3) )*(R/RM-X2)**3
                                                  SUMM2(1) =SUMM2(1)+(EPS/RM)*( (2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(1)/R
                                                  SUMM2(2) =SUMM2(2)+(EPS/RM)*( (2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(2)/R
                                                  SUMM2(3) =SUMM2(3)+(EPS/RM)*( (2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(3)/R

                                                  FCORR2(ANTAL,i,1)=(EPS/RM)*((2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(1)/R

                                                  FCORR2(ANTAL,i,2)=(EPS/RM)*((2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(2)/R

                                                  FCORR2(ANTAL,i,3)=(EPS/RM)*((2*AZ(1)*(R/RM)+AZ(2))*(R/RM-X2)**3+&
                                                  & 3*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**2)*VECT(3)/R

                                                  NAMNCORR2(i) =NAMNCORR2(i)+EPS*(AZ(1)*(R/RM)**2+AZ(2)*(R/RM)+AZ(3))*(R/RM-X2)**3
                                          ENDIF
                                ENDIF
                          ELSE
                              DO vv=7,8
                                IF ( RKK(vv)-R .GE. 0.0d0 ) THEN
                                        NAMNARE2 = NAMNARE2+AKK(vv)*(RKK(vv)-R)**3
                                        SUMM2(1) = SUMM2(1)-3.0d0*AKK(vv)*(VECT(1)/(R))*(RKK(vv)-R)**2
                                        SUMM2(2) = SUMM2(2)-3.0d0*AKK(vv)*(VECT(2)/(R))*(RKK(vv)-R)**2
                                        SUMM2(3) = SUMM2(3)-3.0d0*AKK(vv)*(VECT(3)/(R))*(RKK(vv)-R)**2
                                        FCORR2(ANTAL,i,1)=FCORR2(ANTAL,i,1)-3.0d0*AKK(vv)*(VECT(1)/(R))*(RKK(vv)-R)**2
                                        FCORR2(ANTAL,i,2)=FCORR2(ANTAL,i,2)-3.0d0*AKK(vv)*(VECT(2)/(R))*(RKK(vv)-R)**2
                                        FCORR2(ANTAL,i,3)=FCORR2(ANTAL,i,3)-3.0d0*AKK(vv)*(VECT(3)/(R))*(RKK(vv)-R)**2
                                        NAMNCORR2(i) = NAMNCORR2(i)+AKK(vv)*(RKK(vv)-R)**3
                                 ENDIF
                               ENDDO
                          ENDIF      
                          !print*,NAMNARE1,NAMNARE2
		   ENDIF
                          
                  IF ( POT .EQ. 'ZRP'  .OR. POT .EQ. 'EAM') THEN
                          !Here the cell without displacement:
                          VECT = VECT*LATT
                          R = R*LATT
                          !correctional factors used to transform the
                          !Finnis Sinclair potential (ZRP-pot) into
                          ! an EAM-potential.
                          IF ( POT .EQ. 'EAM' ) THEN
                                  EAM1 = ( 1.0d0 + (X1/X2)*Rm*(R/X2)**(Rm-1) )*exp(-b3*( (R/X2)**Rm -(X1/X2)**Rm) )
                                  EAM2 = ( 1.0d0 + (X1/X2)*Rm*(R/X2)**(Rm-1) )*exp(-2*b4*( (R/X2)**Rm -(X1/X2)**Rm) )
                                  EAM3 = exp(-2*b4*( (R/X2)**Rm -(X1/X2)**Rm) )
                          ELSE
                                  EAM1 = 1.0d0
                                  EAM2 = 1.0d0
                                  EAM3 = 1.0d0
                          ENDIF
                        IF ( R .LT. X2 .OR. POT .EQ. 'EAM') THEN
                          SUM1(1) = SUM1(1)-b1*b3*exp(-b3*(R/X1-1))*VECT(1)/(X1*R)*EAM1
                          SUM1(2) = SUM1(2)-b1*b3*exp(-b3*(R/X1-1))*VECT(2)/(X1*R)*EAM1
                          SUM1(3) = SUM1(3)-b1*b3*exp(-b3*(R/X1-1))*VECT(3)/(X1*R)*EAM1

                          SUM2(1) = SUM2(1)+b4*b2*exp(-2*b4*(R/X1-1))*VECT(1)/(X1*R)*EAM2
                          SUM2(2) = SUM2(2)+b4*b2*exp(-2*b4*(R/X1-1))*VECT(2)/(X1*R)*EAM2
                          SUM2(3) = SUM2(3)+b4*b2*exp(-2*b4*(R/X1-1))*VECT(3)/(X1*R)*EAM2
                          
                          FCORR1(ANTAL,i,1) = b4*b2*exp(-2*b4*(R/X1-1))*VECT(1)/(X1*R)*EAM2
                          FCORR1(ANTAL,i,2) = b4*b2*exp(-2*b4*(R/X1-1))*VECT(2)/(X1*R)*EAM2
                          FCORR1(ANTAL,i,3) = b4*b2*exp(-2*b4*(R/X1-1))*VECT(3)/(X1*R)*EAM2
                          NAMNARE1 = NAMNARE1 + exp(-2*b4*(R/X1-1))*EAM3
                          NAMNCORR1(i) = NAMNCORR1(i) + exp(-2*b4*(R/X1-1))*EAM3
                        ENDIF
                          !Here the cell with displacement:
                          R2 = RP2(NCENTER,k,:)
                          R2(1) = R2(1)+i1*1.0d0
                          R2(2) = R2(2)+i2*1.0d0
                          R2(3) = R2(3)+i3*1.0d0
                          VECT = RR1 - R2
                          VECT = MATMUL(MATR,VECT)*LATT
                          R = sqrt(DOT_PRODUCT(VECT,VECT))
                          IF ( POT .EQ. 'EAM' ) THEN
                                  EAM1 = ( 1.0d0 + (X1/X2)*Rm*(R/X2)**(Rm-1) )*exp(-b3*( (R/X2)**Rm -(X1/X2)**Rm) )
                                  EAM2 = ( 1.0d0 + (X1/X2)*Rm*(R/X2)**(Rm-1) )*exp(-2*b4*( (R/X2)**Rm -(X1/X2)**Rm) )
                                  EAM3 = exp(-2*b4*( (R/X2)**Rm -(X1/X2)**Rm) )
                          ELSE
                                  EAM1 = 1.0d0
                                  EAM2 = 1.0d0
                                  EAM3 = 1.0d0
                          ENDIF
                        IF  ( R .LT. X2 .OR. POT .EQ. 'EAM' ) THEN
                          SUMM1(1) = SUMM1(1)-b1*b3*exp(-b3*(R/X1-1))*VECT(1)/(X1*R)*EAM1
                          SUMM1(2) = SUMM1(2)-b1*b3*exp(-b3*(R/X1-1))*VECT(2)/(X1*R)*EAM1
                          SUMM1(3) = SUMM1(3)-b1*b3*exp(-b3*(R/X1-1))*VECT(3)/(X1*R)*EAM1

                          SUMM2(1) = SUMM2(1)+b4*b2*exp(-2*b4*(R/X1-1))*VECT(1)/(X1*R)*EAM2
                          SUMM2(2) = SUMM2(2)+b4*b2*exp(-2*b4*(R/X1-1))*VECT(2)/(X1*R)*EAM2
                          SUMM2(3) = SUMM2(3)+b4*b2*exp(-2*b4*(R/X1-1))*VECT(3)/(X1*R)*EAM2
                          
                          FCORR2(ANTAL,i,1) = b4*b2*exp(-2*b4*(R/X1-1))*VECT(1)/(X1*R)*EAM2
                          FCORR2(ANTAL,i,2) = b4*b2*exp(-2*b4*(R/X1-1))*VECT(2)/(X1*R)*EAM2
                          FCORR2(ANTAL,i,3) = b4*b2*exp(-2*b4*(R/X1-1))*VECT(3)/(X1*R)*EAM2

                          NAMNARE2 = NAMNARE2 + exp(-2*b4*(R/X1-1))*EAM3
                          NAMNCORR2(i) = NAMNCORR2(i) + exp(-2*b4*(R/X1-1))*EAM3
                        ENDIF
                  ENDIF
                ENDIF
                ENDIF
             ENDDO
           ENDDO
           ENDDO
           ENDDO
           IF ( POT .EQ. 'MSV' ) THEN
             F1(i,:) = SUM1*(EPS/RM)
             F2(i,:) = SUMM1*(EPS/RM)
           ENDIF
           IF ( POT .EQ. 'BUC' ) THEN
             rat = (EPS/d)*(6.0*SIGMA/(SIGMA-6.0))
             F1(i,:) = rat*(SUM1 - SUM2)
             F2(i,:) = rat*(SUMM1 - SUMM2)
           ENDIF
           IF ( POT .EQ. 'PAS' ) THEN
                   SUM2 = 2.0d0*(NAMNARE1-X2)*SUM2
                   SUMM2 = 2.0d0*(NAMNARE2-X2)*SUMM2
                   F1(i,:)  =  -SUM1+SUM2
                   F2(i,:)  =  -SUMM1+SUMM2
           ENDIF
           IF ( POT .EQ. 'PAA' ) THEN
                   DS = NAMNARE1
                   SUM2 = ( 8*B1*DS**7 + 7*B2*DS**6 + 6*B3*DS**5 + 5*B4*DS**4 + 4*C6*DS**3 + 3*C8*DS**2 + 2*C10*DS + AZ(4) )*SUM2
                   DS = NAMNARE2
                   SUMM2 = ( 8*B1*DS**7 + 7*B2*DS**6 + 6*B3*DS**5 + 5*B4*DS**4 + 4*C6*DS**3 + 3*C8*DS**2 + 2*C10*DS + AZ(4) )*SUMM2
                   F1(i,:)  =  -2.0d0*SUM1-SUM2
                   F2(i,:)  =  -2.0d0*SUMM1-SUMM2
           ENDIF
           IF ( POT .EQ. 'PIN' ) THEN
                   F1(i,:) = -SUM1+0.50d0*SUM2/sqrt(NAMNARE1)
                   F2(i,:) = -SUMM1+0.50d0*SUMM2/sqrt(NAMNARE2)
           ENDIF
           IF ( POT .EQ. 'ZRP' .OR. POT .EQ. 'EAM' ) THEN
                IF ( POT .EQ. 'ZRP' ) THEN
                        F1(i,:) = -2.0d0*(SUM1 + 0.50d0*SUM2/SQRT(NAMNARE1))
                        F2(i,:) = -2.0d0*(SUMM1 + 0.50d0*SUMM2/SQRT(NAMNARE2))
                ELSE
                        F1(i,:) = -(1.0d0/4.0d0)*(SUM1 - 0.5d0*SUM2/SQRT(NAMNARE1))
                        F2(i,:) = -(1.0d0/4.0d0)*(SUMM1 - 0.5d0*SUMM2/SQRT(NAMNARE2))
                ENDIF
                   
            ENDIF
       ENDDO
        IF ( POT .EQ. 'ZRP' .OR. POT .EQ. 'EAM' .OR. POT .EQ. 'PIN' ) THEN
                IF ( POT .EQ. 'ZRP' ) FACT = -1.0d0
                IF ( POT .EQ. 'EAM' ) FACT = (1.0d0/8.0d0)
                IF ( POT .EQ. 'PIN' ) FACT = 0.50d0
                DO i =1,NVECT
                   DO j=1,ANTAL
                        F1(i,:) = F1(i,:)+FACT*FCORR1(j,i,:)/sqrt(NAMNCORR1(INDUX(j)))
                        F2(i,:) = F2(i,:)+FACT*FCORR2(j,i,:)/sqrt(NAMNCORR2(INDUX(j)))
                   ENDDO
                ENDDO   
        ENDIF

        IF ( POT .EQ. 'PAA' ) THEN
                DO i =1,NVECT
                 DO j=1,ANTAL
                  DS1 = NAMNCORR1(INDUX(j))
                  DS2 = NAMNCORR2(INDUX(j))
                  F1(i,:) = F1(i,:)-( 8*B1*DS1**7 + 7*B2*DS1**6 + 6*B3*DS1**5 +5*B4*DS1**4 + 4*C6*DS1**3 + 3*C8*DS1**2 + 2*C10*DS1 + AZ(4) )*&
                  & FCORR1(j,i,:)
                  F2(i,:) = F2(i,:)-( 8*B1*DS2**7 + 7*B2*DS2**6 + 6*B3*DS2**5 +5*B4*DS2**4 + 4*C6*DS2**3 + 3*C8*DS2**2 + 2*C10*DS2 + AZ(4) )*&
                  & FCORR2(j,i,:)
                ENDDO
               ENDDO
        ENDIF
                  
       ! Here the difference in forces are calculated

       IF (ITTER .EQ. 0 ) FORCE =  F2(:,:) - F1(:,:)
       IF (ITTER .NE. 0 ) FORCE =  F1(:,:) - F2(:,:)
       FORCES = FORCE
END SUBROUTINE KRAFTER
